@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2015 David R. MacIver
+# Most of this work is copyright (C) 2013-2017 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -28,7 +28,7 @@ import os
 import inspect
 import warnings
 import threading
-from enum import Enum, unique, IntEnum
+from enum import Enum, IntEnum, unique
 from collections import namedtuple
 
 from hypothesis.errors import InvalidArgument, HypothesisDeprecationWarning
@@ -49,8 +49,9 @@ _db_cache = {}
 
 class settingsProperty(object):
 
-    def __init__(self, name):
+    def __init__(self, name, show_default):
         self.name = name
+        self.show_default = show_default
 
     def __get__(self, obj, type=None):
         if obj is None:
@@ -65,18 +66,18 @@ class settingsProperty(object):
         obj.__dict__[self.name] = value
 
     def __delete__(self, obj):
-        try:
-            del obj.__dict__[self.name]
-        except KeyError:
-            raise AttributeError(self.name)
+        raise AttributeError('Cannot delete attribute %s' % (self.name,))
 
     @property
     def __doc__(self):
+        default = repr(getattr(settings.default, self.name)) if \
+            self.show_default else '(dynamically calculated)'
         return '\n'.join((
             all_settings[self.name].description,
-            'default value: %r' % (getattr(settings.default, self.name),)
+            'default value: %s' % (default,)
         ))
 
+
 default_variable = DynamicVariable(None)
 
 
@@ -91,17 +92,13 @@ class settingsMeta(type):
         if v is not None:
             return v
         if hasattr(settings, '_current_profile'):
-            default_variable.value = settings.load_profile(
-                settings._current_profile
-            )
+            settings.load_profile(settings._current_profile)
             assert default_variable.value is not None
         return default_variable.value
 
     @default.setter
     def default(self, value):
-        if default_variable.value is not None:
-            raise AttributeError('Cannot assign settings.default')
-        self._assign_default_internal(value)
+        raise AttributeError('Cannot assign settings.default')
 
     def _assign_default_internal(self, value):
         default_variable.value = value
@@ -140,7 +137,7 @@ class settings(settingsMeta('settings', (object,), {})):
     ):
         self._construction_complete = False
         self._database = kwargs.pop('database', not_set)
-        explicit_kwargs = list(kwargs)
+        database_file = kwargs.get('database_file', not_set)
         defaults = parent or settings.default
         if defaults is not None:
             for setting in all_settings.values():
@@ -149,7 +146,7 @@ class settings(settingsMeta('settings', (object,), {})):
                 elif setting.validator:
                     kwargs[setting.name] = setting.validator(
                         kwargs[setting.name])
-            if self._database is not_set:
+            if self._database is not_set and database_file is not_set:
                 self._database = defaults.database
         for name, value in kwargs.items():
             if name not in all_settings:
@@ -158,10 +155,6 @@ class settings(settingsMeta('settings', (object,), {})):
             setattr(self, name, value)
         self.storage = threading.local()
         self._construction_complete = True
-        for k in explicit_kwargs:
-            deprecation = all_settings[k].deprecation
-            if deprecation:
-                note_deprecation(deprecation, self)
 
     def defaults_stack(self):
         try:
@@ -177,7 +170,7 @@ class settings(settingsMeta('settings', (object,), {})):
     @classmethod
     def define_setting(
         cls, name, description, default, options=None, deprecation=None,
-        validator=None,
+        validator=None, show_default=True,
     ):
         """Add a new setting.
 
@@ -196,17 +189,12 @@ class settings(settingsMeta('settings', (object,), {})):
             )
         if options is not None:
             options = tuple(options)
-            if default not in options:
-                raise InvalidArgument(
-                    'Default value %r is not in options %r' % (
-                        default, options
-                    )
-                )
+            assert default in options
 
         all_settings[name] = Setting(
-            name, description.strip(), default, options, deprecation, validator
+            name, description.strip(), default, options, validator
         )
-        setattr(settings, name, settingsProperty(name))
+        setattr(settings, name, settingsProperty(name, show_default))
 
     @classmethod
     def lock_further_definitions(cls):
@@ -216,13 +204,11 @@ class settings(settingsMeta('settings', (object,), {})):
         if name in settings._WHITELISTED_REAL_PROPERTIES:
             return object.__setattr__(self, name, value)
         elif name == 'database':
-            if self._construction_complete:
-                raise AttributeError(
-                    'settings objects are immutable and may not be assigned to'
-                    ' after construction.'
-                )
-            else:
-                return object.__setattr__(self, '_database', value)
+            assert self._construction_complete
+            raise AttributeError(
+                'settings objects are immutable and may not be assigned to'
+                ' after construction.'
+            )
         elif name in all_settings:
             if self._construction_complete:
                 raise AttributeError(
@@ -264,20 +250,15 @@ class settings(settingsMeta('settings', (object,), {})):
         this property is accessed on a particular thread.
 
         """
-        try:
-            if self._database is not_set and self.database_file is not None:
-                from hypothesis.database import ExampleDatabase
-                if self.database_file not in _db_cache:
-                    _db_cache[self.database_file] = (
-                        ExampleDatabase(self.database_file))
-                return _db_cache[self.database_file]
-            if self._database is not_set:
-                self._database = None
-            return self._database
-        except AttributeError:
-            import traceback
-            traceback.print_exc()
-            assert False
+        if self._database is not_set and self.database_file is not None:
+            from hypothesis.database import ExampleDatabase
+            if self.database_file not in _db_cache:
+                _db_cache[self.database_file] = (
+                    ExampleDatabase(self.database_file))
+            return _db_cache[self.database_file]
+        if self._database is not_set:
+            self._database = None
+        return self._database
 
     def __enter__(self):
         default_context_manager = default_variable.with_value(self)
@@ -334,7 +315,7 @@ class settings(settingsMeta('settings', (object,), {})):
 
 Setting = namedtuple(
     'Setting', (
-        'name', 'description', 'default', 'options', 'deprecation', 'validator'
+        'name', 'description', 'default', 'options', 'validator'
     ))
 
 
@@ -445,6 +426,7 @@ settings.define_setting(
         os.getenv('HYPOTHESIS_DATABASE_FILE') or
         os.path.join(hypothesis_home_dir(), 'examples')
     ),
+    show_default=False,
     description="""
     database: An instance of hypothesis.database.ExampleDatabase that will be
 used to save examples to and load previous examples from. May be None
@@ -471,6 +453,13 @@ class HealthCheck(Enum):
     return_value = 5
 
 
+@unique
+class Statistics(IntEnum):
+    never = 0
+    interesting = 1
+    always = 2
+
+
 class Verbosity(object):
 
     def __repr__(self):
@@ -510,6 +499,7 @@ class Verbosity(object):
             return result
         raise InvalidArgument('No such verbosity level %r' % (key,))
 
+
 Verbosity.quiet = Verbosity('quiet', 0)
 Verbosity.normal = Verbosity('normal', 1)
 Verbosity.verbose = Verbosity('verbose', 2)
@@ -521,7 +511,7 @@ Verbosity.all = [
 
 ENVIRONMENT_VERBOSITY_OVERRIDE = os.getenv('HYPOTHESIS_VERBOSITY_LEVEL')
 
-if ENVIRONMENT_VERBOSITY_OVERRIDE:
+if ENVIRONMENT_VERBOSITY_OVERRIDE:  # pragma: no cover
     DEFAULT_VERBOSITY = Verbosity.by_name(ENVIRONMENT_VERBOSITY_OVERRIDE)
 else:
     DEFAULT_VERBOSITY = Verbosity.normal
@@ -574,6 +564,14 @@ settings.define_setting(
     description="""A list of health checks to disable"""
 )
 
+settings.define_setting(
+    'report_statistics',
+    default=Statistics.interesting,
+    description=u"""
+If set to True, Hypothesis will run a preliminary health check before
+attempting to actually execute your test.
+"""
+)
 
 settings.lock_further_definitions()
 
