@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2016 David R. MacIver
+# Most of this work is copyright (C) 2013-2017 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -24,7 +24,7 @@ import pytest
 from hypothesis.reporting import default as default_reporter
 from hypothesis.reporting import with_reporter
 from hypothesis.statistics import collector
-from hypothesis.internal.compat import text_type, OrderedDict
+from hypothesis.internal.compat import OrderedDict, text_type
 from hypothesis.internal.detection import is_hypothesis_test
 
 PYTEST_VERSION = tuple(map(
@@ -68,10 +68,13 @@ if PYTEST_VERSION >= (2, 7, 0):
         profile = config.getoption(LOAD_PROFILE_OPTION)
         if profile:
             settings.load_profile(profile)
+        config.addinivalue_line(
+            'markers',
+            'hypothesis: Tests which use hypothesis.')
 
     gathered_statistics = OrderedDict()
 
-    @pytest.hookimpl(hookwrapper=True)
+    @pytest.mark.hookwrapper
     def pytest_runtest_call(item):
         if not (hasattr(item, 'obj') and is_hypothesis_test(item.obj)):
             yield
@@ -87,7 +90,7 @@ if PYTEST_VERSION >= (2, 7, 0):
             if store.results:
                 item.hypothesis_report_information = list(store.results)
 
-    @pytest.hookimpl(hookwrapper=True)
+    @pytest.mark.hookwrapper
     def pytest_runtest_makereport(item, call):
         report = (yield).get_result()
         if hasattr(item, 'hypothesis_report_information'):
