@@ -1,12 +1,13 @@
 # coding=utf-8
 #
-# This file is part of Hypothesis (https://github.com/DRMacIver/hypothesis)
+# This file is part of Hypothesis, which may be found at
+# https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2015 David R. MacIver
+# Most of this work is copyright (C) 2013-2017 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
-# https://github.com/DRMacIver/hypothesis/blob/master/CONTRIBUTING.rst for a
-# full list of people who may hold copyright, and consult the git log if you
-# need to determine who owns an individual contribution.
+# CONTRIBUTING.rst for a full list of people who may hold copyright, and
+# consult the git log if you need to determine who owns an individual
+# contribution.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
@@ -17,34 +18,31 @@
 from __future__ import division, print_function, absolute_import
 
 from hypothesis import settings
-from hypothesis.internal.compat import hrange, getargspec, \
-    unicode_safe_repr
+from hypothesis.internal.compat import hrange, getargspec
 from hypothesis.internal.reflection import arg_string, \
     convert_keyword_arguments, convert_positional_arguments
 from hypothesis.searchstrategy.strategies import SearchStrategy
 
 
-class reprmangledtuple(tuple):
-
-    def __repr__(self):
-        try:
-            return super(reprmangledtuple, self).__repr__()
-        except UnicodeEncodeError:  # pragma: no cover
-            if len(self) == 1:
-                return u"(%s,)" % (unicode_safe_repr(self[0]),)
-            else:
-                return u"(%s)" % (u", ".join(
-                    map(unicode_safe_repr, self)
-                ))
-
-
 def tupleize(x):
     if isinstance(x, (tuple, list)):
-        return reprmangledtuple(x)
+        return tuple(x)
     else:
         return x
 
 
+def unwrap_strategies(s):
+    if isinstance(s, SearchStrategy):
+        while True:
+            assert isinstance(s, SearchStrategy)
+            try:
+                s = s.wrapped_strategy
+            except AttributeError:
+                return s
+    else:
+        return s
+
+
 class DeferredStrategy(SearchStrategy):
 
     """A strategy which is defined purely by conversion to and from another
@@ -65,22 +63,29 @@ class DeferredStrategy(SearchStrategy):
         )
         self.__settings = settings.default or settings()
 
+    @property
+    def supports_find(self):
+        return self.wrapped_strategy.supports_find
+
+    @property
+    def is_empty(self):
+        return self.wrapped_strategy.is_empty
+
     @property
     def wrapped_strategy(self):
         if self.__wrapped_strategy is None:
             with self.__settings:
                 self.__wrapped_strategy = self.__function(
-                    *self.__args,
-                    **self.__kwargs
-                )
+                    *[unwrap_strategies(s) for s in self.__args],
+                    **{k: unwrap_strategies(v)
+                       for k, v in self.__kwargs.items()})
         return self.__wrapped_strategy
 
     def validate(self):
-        self.wrapped_strategy.validate()
-
-    @property
-    def template_upper_bound(self):
-        return self.wrapped_strategy.template_upper_bound
+        w = self.wrapped_strategy
+        assert isinstance(w, SearchStrategy), \
+            '%r returned non-strategy %r' % (self, w)
+        w.validate()
 
     def __repr__(self):
         if self.__representation is None:
@@ -101,30 +106,12 @@ class DeferredStrategy(SearchStrategy):
             for k, v in defaults.items():
                 if k in kwargs_for_repr and kwargs_for_repr[k] is defaults[k]:
                     del kwargs_for_repr[k]
-            self.__representation = u'%s(%s)' % (
+            self.__representation = '%s(%s)' % (
                 self.__function.__name__,
                 arg_string(
                     self.__function, _args, kwargs_for_repr, reorder=False),
             )
         return self.__representation
 
-    def draw_parameter(self, random):
-        return self.wrapped_strategy.draw_parameter(random)
-
-    def draw_template(self, random, pv):
-        return self.wrapped_strategy.draw_template(random, pv)
-
-    def reify(self, value):
-        return self.wrapped_strategy.reify(value)
-
-    def simplifiers(self, random, template):
-        return self.wrapped_strategy.simplifiers(random, template)
-
-    def strictly_simpler(self, x, y):
-        return self.wrapped_strategy.strictly_simpler(x, y)
-
-    def to_basic(self, template):
-        return self.wrapped_strategy.to_basic(template)
-
-    def from_basic(self, data):
-        return self.wrapped_strategy.from_basic(data)
+    def do_draw(self, data):
+        return data.draw(self.wrapped_strategy)
